@@ -7,8 +7,8 @@
 #' @name BivariateFitters
 #' @aliases BivariateFitters BivariateFitter
 #' @description
-#' These are computing engines called by \code{\link{NGeDS}}, needed for the
-#' underlying fitting procedures.
+#' These are computing engines called by \code{\link{NGeDS}} and
+#' \code{\link{GGeDS}}, needed for the underlying fitting procedures.
 #' @param X a numeric vector containing \eqn{N} sample values of the first
 #' independent variable chosen to enter the spline regression component of the
 #' predictor model.
@@ -30,7 +30,8 @@
 #' \code{\link[stats]{gaussian}}) or the result of a call to a family function
 #' (e.g. \code{gaussian()}). See \link[stats]{family} for details on family
 #' functions.
-#' @param Indicator contingency table of \code{X} and \code{Y}.
+#' @param Indicator contingency table (i.e., frequency of observations) for the
+#' independent variables \code{X} and \code{Y}.
 #' @param beta numeric parameter in the interval \eqn{[0,1]} tuning the knot
 #' placement in stage A of GeDS. See the description of \code{\link{NGeDS}} or
 #' \code{\link{GGeDS}}.
@@ -38,12 +39,12 @@
 #' threshold for the stopping rule  (model selector) in stage A of GeDS. See
 #' also \code{stoptype} and details in the description of \code{\link{NGeDS}} or
 #' \code{\link{GGeDS}}.
-#' @param min.intknots optional parameter allowing the user to set a minimum
-#' number of internal knots required. By default equal to zero.
+#' @param min.intknots optional parameter specifying the minimum number of
+#' internal knots required in Stage A's fit. Default is zero.
 #' @param max.intknots optional parameter allowing the user to set a maximum
-#' number of internal knots to be added by the GeDS estimation algorithm. By
-#' default equal to the number of internal knots \eqn{\kappa} for the saturated
-#' GeDS model (i.e. \eqn{\kappa=N-2}).
+#' number of internal knots to be added in Stage A by the GeDS estimation
+#' algorithm. Default equals the number of internal knots \eqn{\kappa} for the
+#' saturated GeDS model (i.e. \eqn{\kappa=N-2}).
 #' @param q numeric parameter which allows to fine-tune the stopping rule of
 #' stage A of GeDS, by default equal to 2. See details in the description of
 #' \code{\link{NGeDS}} or \code{\link{GGeDS}}.
@@ -51,8 +52,8 @@
 #' the range of \code{X}.
 #' @param Yextr boundary knots in the \code{Y} direction. By default equal to
 #' the range of \code{Y}.
-#' @param show.iters logical variable indicating whether or not to print 
-#' information at each step. By default equal to \code{FALSE}.
+#' @param show.iters logical variable indicating whether or not to print fitting
+#' information at each step. Default is \code{FALSE}.
 #' @param stoptype a character string indicating the type of GeDS stopping rule
 #' to be used. It should be either \code{"SR"}, \code{"RD"} or \code{"LR"},
 #' partial match allowed. See details of \code{\link{NGeDS}} or
@@ -60,18 +61,22 @@
 #' @param tol numeric value indicating the tolerance to be used in checking
 #' whether two knots should be considered different during the knot placement
 #' steps in stage A.
-#' @param higher_order a logical that defines whether to compute the higher
+#' @param higher_order a logical defining whether to compute the higher
 #' order fits (quadratic and cubic) after stage A is run. Default is
 #' \code{TRUE}.
-#' @param Xintknots vector of starting internal knots in the \code{X} direction.
-#' Default is \code{NULL}.
-#' @param Yintknots vector of starting internal knots in the \code{Y} direction.
-#' Default is \code{NULL}.
+#' @param Xintknots a vector of starting internal knots in the \code{X} direction. 
+#' Allows the user to begin Stage A's GeDS algorithm with a linear spline fit
+#' using a predefined vector of internal \code{X} knots, instead of starting with
+#' a straight line fit. Default is \code{NULL}.
+#' @param Yintknots a vector of starting internal knots in the \code{Y} direction. 
+#' Allows the user to begin Stage A's GeDS algorithm with a linear spline fit
+#' using a predefined vector of internal \code{X} knots, instead of starting with
+#' a straight line fit. Default is \code{NULL}.
 #' 
 #' @return A \code{\link{GeDS-Class}} object, but without the \code{Formula},
 #' \code{extcall}, \code{terms} and \code{znames} slots.
 #' 
-#' @seealso \code{\link{NGeDS}} and \code{\link{UnivariateFitters}}.
+#' @seealso \code{\link{NGeDS}}, \code{\link{GGeDS}} and \code{\link{UnivariateFitters}}.
 #' 
 #' @export
 #' @rdname BivariateFitters
@@ -344,12 +349,12 @@ BivariateFitter <- function(X, Y, Z, W, weights = rep(1,length(X)), Indicator,
   
   # Keep the non-NA columns from the "j"th row
   toBeSaved <- sum(!is.na(previousX[j,]))
-  previousX <- previousX[ ,-((toBeSaved + 1):max(max.intknots + 4, toBeSaved + 1))]
+  previousX <- previousX[ ,-((toBeSaved + 1):max(max.intknots + 4, toBeSaved + 1)), drop = FALSE]
   toBeSaved <- sum(!is.na(previousY[j,]))
-  previousY <- previousY[ ,-((toBeSaved + 1):max(max.intknots + 4, toBeSaved + 1))]
+  previousY <- previousY[ ,-((toBeSaved + 1):max(max.intknots + 4, toBeSaved + 1)), drop = FALSE]
   
   # Keep the corresponding (intknotsX + 2) * (intknotsY + 2) coefficients
-  oldcoef <- oldcoef[, 1:((NCOL(previousX) - 4 + 2) * (NCOL(previousY) - 4 + 2))]
+  oldcoef <- oldcoef[, 1:((NCOL(previousX) - 4 + 2) * (NCOL(previousY) - 4 + 2)), drop = FALSE]
   
   if (j == max.intknots + 1) {
     warning("Maximum number of iterations exceeded")
@@ -358,9 +363,9 @@ BivariateFitter <- function(X, Y, Z, W, weights = rep(1,length(X)), Indicator,
     iter <- j
     } else {
       # Delete from the "j+1th" row until the "max.intknots+1th" row (i.e. keep the j first rows)
-      previousX <- previousX[-((j+1):(max.intknots+1)), ] 
-      previousY <- previousY[-((j+1):(max.intknots+1)), ]
-      oldcoef   <- oldcoef[-((j+1):(max.intknots+1)),]
+      previousX <- previousX[-((j+1):(max.intknots+1)), , drop = FALSE] 
+      previousY <- previousY[-((j+1):(max.intknots+1)), , drop = FALSE]
+      oldcoef   <- oldcoef[-((j+1):(max.intknots+1)), , drop = FALSE]
       
       lastXknots <- sum(!is.na(previousX[j-q, ]))
       lastYknots <- sum(!is.na(previousY[j-q, ]))
@@ -415,7 +420,7 @@ BivariateFitter <- function(X, Y, Z, W, weights = rep(1,length(X)), Indicator,
   
   out <- list("Type" = "LM - Biv", "Linear.IntKnots" = list("Xk" = llX, "Yk" = llY), "Quadratic.IntKnots" = list("Xk" = qqX, "Yk" = qqY),
               "Cubic.IntKnots" = list("Xk" = ccX,"Yk" = ccY),"Dev.Linear" = lin$RSS, "Dev.Quadratic" = squ$RSS, "Dev.Cubic" = cub$RSS,
-              "RSS" = RSSnew, "Linear" = lin, "Quadratic" = squ, "Cubic" = cub, "Stored" = list("previousX" = previousX, "previousY" = previousY),
+              "RSS" = RSSnew, "Linear.Fit" = lin, "Quadratic.Fit" = squ, "Cubic.Fit" = cub, "Stored" = list("previousX" = previousX, "previousY" = previousY),
               "Args" = args, "Call" = save, "Nintknots" = list("X" = length(llX), "Y" = length(llY)), "iters" = j, "Guesses" = NULL,
               "Coefficients" = oldcoef)
   class(out) <- "GeDS"
@@ -724,12 +729,12 @@ GenBivariateFitter <- function(X, Y, Z, W, family = family, weights = rep(1,leng
   
   # Keep the non-NA columns from the "j"th row
   toBeSaved <- sum(!is.na(previousX[j,]))
-  previousX <- previousX[ ,-((toBeSaved + 1):max(max.intknots + 4, toBeSaved + 1))]
+  previousX <- previousX[ ,-((toBeSaved + 1):max(max.intknots + 4, toBeSaved + 1)), drop = FALSE]
   toBeSaved <- sum(!is.na(previousY[j,]))
-  previousY <- previousY[ ,-((toBeSaved + 1):max(max.intknots + 4, toBeSaved + 1))]
+  previousY <- previousY[ ,-((toBeSaved + 1):max(max.intknots + 4, toBeSaved + 1)), drop = FALSE]
   
   # Keep the corresponding (intknotsX + 2) * (intknotsY + 2) coefficients
-  oldcoef <- oldcoef[, 1:((NCOL(previousX) - 4 + 2) * (NCOL(previousY) - 4 + 2))]
+  oldcoef <- oldcoef[, 1:((NCOL(previousX) - 4 + 2) * (NCOL(previousY) - 4 + 2)), drop = FALSE]
   
   if (j == max.intknots + 1) {
     warning("Maximum number of iterations exceeded")
@@ -738,9 +743,9 @@ GenBivariateFitter <- function(X, Y, Z, W, family = family, weights = rep(1,leng
     iter <- j
   } else {
     # Delete from the "j+1th" row until the "max.intknots+1th" row (i.e. keep the j first rows)
-    previousX <- previousX[-((j+1):(max.intknots+1)), ] 
-    previousY <- previousY[-((j+1):(max.intknots+1)), ]
-    oldcoef   <- oldcoef[-((j+1):(max.intknots+1)),]
+    previousX <- previousX[-((j+1):(max.intknots+1)), , drop = FALSE] 
+    previousY <- previousY[-((j+1):(max.intknots+1)), , drop = FALSE]
+    oldcoef   <- oldcoef[-((j+1):(max.intknots+1)), , drop = FALSE]
     
     lastXknots <- sum(!is.na(previousX[j-q, ]))
     lastYknots <- sum(!is.na(previousY[j-q, ]))
@@ -812,7 +817,7 @@ GenBivariateFitter <- function(X, Y, Z, W, family = family, weights = rep(1,leng
   
   out <- list("Type" = "GLM - Biv", "Linear.IntKnots" = list("Xk" = llX, "Yk" = llY), "Quadratic.IntKnots" = list("Xk" = qqX, "Yk" = qqY),
               "Cubic.IntKnots" = list("Xk" = ccX, "Yk" = ccY), "Dev.Linear" = lin$RSS, "Dev.Quadratic" = squ$RSS, "Dev.Cubic" = cub$RSS,
-              "RSS" = RSSnew, "Linear" = lin, "Quadratic" = squ, "Cubic" = cub, "Stored" = list("previousX" = previousX, "previousY" = previousY),
+              "RSS" = RSSnew, "Linear.Fit" = lin, "Quadratic.Fit" = squ, "Cubic.Fit" = cub, "Stored" = list("previousX" = previousX, "previousY" = previousY),
               "Args"= args, "Call" = save, "Nintknots" = list("X"= length(llX), "Y"= length(llY)), "iters" = j, "Guesses" = NULL,
               "Coefficients" = oldcoef)
   class(out) <- "GeDS"
@@ -900,7 +905,8 @@ placeKnot <- function(Dim, Dim.intknots, matr, Indicator, FixedDim, ordFixedDim,
   }
   # (Step 4 - UnivariateFitter) Calculate the normalized within-cluster means and ranges 
   Dim.mean  <- Dim.mean/max(Dim.mean)
-  Dim.width <- Dim.width/max(Dim.width)
+  # If the residual clusters are all singletons then all the Dim.widths will equal 0, and we cannot divide by 0
+  if (max(Dim.width) != 0) Dim.width <- Dim.width/max(Dim.width)
   # Calculate the cluster weights (Step 5 - UnivariateFitter)
   Dim.weights <- beta*Dim.mean + (1 - beta)*Dim.width
   
